package net.tomatentum.cutin.container;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import net.tomatentum.cutin.ReflectedMethodFactory;
import net.tomatentum.cutin.method.ReflectedMethod;

public class LoneMethodContainer<I extends Object, C extends Object> implements MethodContainer<I, C> {

    private Map<I, ReflectedMethod<I, C>> methodStore;
    private ReflectedMethodFactory<I, C> factory;

    public LoneMethodContainer(ReflectedMethodFactory<I, C> factory) {
        this.methodStore = new HashMap<>();
        this.factory = factory;
    }

    @Override
    public MethodContainer<I, C> addMethod(ReflectedMethod<I, C> method) {
        this.methodStore.put(method.identifier(), method);
        return this;
    }

    @Override
    public MethodContainer<I, C> addMethods(Object containingObject, Method... methods) {
        for (Method method : methods)
            this.factory.produce(method, containingObject)
                .ifPresent(this::addMethod);
        return this;
    }

    @Override
    public Set<I> identifiers() {
        return methodStore.keySet();
    }

    @Override
    public Collection<ReflectedMethod<I, C>> methods() {
        return this.methodStore.values();
    }

    @Override
    public Collection<ReflectedMethod<I, C>> findFor(I identifier) {
        return Arrays.asList(this.methodStore.get(identifier));
    }

    @Override
    public Optional<ReflectedMethod<I, C>> findFirstFor(I identifier) {
        return findFor(identifier).stream().findFirst();
    }

}
